PROTO = {
    'spaceone.inventory.interface.grpc.v1.region': ['Region'],
    'spaceone.inventory.interface.grpc.v1.collector': ['Collector'],
    'spaceone.inventory.interface.grpc.v1.job': ['Job'],
    'spaceone.inventory.interface.grpc.v1.job_task': ['JobTask'],
    'spaceone.inventory.interface.grpc.v1.cloud_service_type': ['CloudServiceType'],
    'spaceone.inventory.interface.grpc.v1.cloud_service': ['CloudService'],
    'spaceone.inventory.interface.grpc.v1.resource_group': ['ResourceGroup'],
    'spaceone.inventory.interface.grpc.v1.change_history': ['ChangeHistory'],
    'spaceone.inventory.interface.grpc.v1.note': ['Note'],
    'spaceone.inventory.interface.grpc.v1.collector_rule': ['CollectorRule'],
    'spaceone.inventory.interface.grpc.v1.cloud_service_query_set': ['CloudServiceQuerySet'],
    'spaceone.inventory.interface.grpc.v1.cloud_service_stats': ['CloudServiceStats']
}
